<?php
namespace Setka\Editor\Service\AMP\AnimationSanitizer;

class AnimatedDOMElement implements AnimatedDOMElementInterface
{
    /**
     * @var \DOMElement
     */
    private $element;

    /**
     * AnimatedDOMElement constructor.
     *
     * @param \DOMElement $element
     */
    public function __construct(\DOMElement $element)
    {
        $this->element = $element;
    }

    public function __call($name, $arguments)
    {
        return call_user_func_array(array($this->element, $name), $arguments);
    }

    /**
     * @inheritdoc
     */
    public function getOpacity()
    {
        return (float) $this->element->getAttribute('data-anim-opacity') / 100;
    }

    /**
     * @inheritdoc
     */
    public function getRotate()
    {
        return (float) $this->element->getAttribute('data-anim-rotation') . 'deg';
    }

    /**
     * @inheritdoc
     */
    public function getTranslate()
    {
        $direction = $this->element->getAttribute('data-anim-direction');
        $shift     = (int) $this->element->getAttribute('data-anim-shift');

        if ('bottom' === $direction || 'right' === $direction) {
            $shift = $shift * -1;
        }

        $shift = (string) $shift . 'px';
        $zero  = '0px';

        if ('top' === $direction || 'bottom' === $direction) {
            $translate = array($zero, $shift);
        } else {
            $translate = array($shift, $zero);
        }

        return $translate;
    }

    /**
     * @inheritdoc
     */
    public function getScale()
    {
        return (float) $this->element->getAttribute('data-anim-zoom') / 100;
    }

    /**
     * @inheritdoc
     */
    public function getAnimationTrigger()
    {
        return $this->element->getAttribute('data-anim-trigger');
    }

    /**
     * @return bool
     */
    public function isTriggersByScroll()
    {
        return 'whenScrolling' === $this->getAnimationTrigger();
    }

    /**
     * @inheritdoc
     */
    public function getAnimationScrollStart()
    {
        return (int) $this->element->getAttribute('data-anim-scroll-start');
    }

    /**
     * @inheritdoc
     */
    public function getAnimationScrollFinish()
    {
        return (int) $this->element->getAttribute('data-anim-scroll-finish');
    }

    /**
     * @inheritdoc
     */
    public function getAnimationDuration()
    {
        if ($this->isTriggersByScroll()) {
            return 1000.0;
        }
        return (float) $this->element->getAttribute('data-anim-duration') * 1000;
    }

    /**
     * @inheritdoc
     */
    public function getAnimationDelay()
    {
        if ($this->isTriggersByScroll()) {
            return 0.0;
        }
        return (float) $this->element->getAttribute('data-anim-delay') * 1000;
    }
}
