<?php
namespace Setka\Editor\Service\SystemReport;

abstract class AbstractSection implements SectionInterface
{
    /**
     * @var string
     */
    protected $title;

    /**
     * @var mixed
     */
    protected $value;

    /**
     * @var array
     */
    protected $buildMethods = array();

    /**
     * @var \Exception[]
     */
    protected $errors = array();

    /**
     * @var boolean
     */
    protected $isChecked = false;

    /**
     * @inheritDoc
     */
    abstract public function build();

    /**
     * @inheritDoc
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @inheritDoc
     */
    public function getValue()
    {
        return $this->value;
    }

    /**
     * @param array $functions
     */
    private function checkFunctions(array $functions)
    {
        foreach ($functions as $function) {
            if (is_array($function)) {
                $this->checkClassFunction($function);
            } else {
                $this->checkFunction($function);
            }
        }
    }

    /**
     * @param string $function
     *
     * @return $this
     */
    private function checkFunction($function)
    {
        if (function_exists($function)) {
            return $this;
        }
        $this->errors[] = new \Exception('Function "' . $function . '" disabled or not exists in current PHP interpreter.');
    }

    /**
     * @param array $callback
     *
     * @return $this
     */
    private function checkClassFunction(array $callback)
    {
        if (in_array($callback[1], get_class_methods($callback[0]), true)) {
            return $this;
        }
        $this->errors[] = new \Exception('Function "' . implode('::', $callback) . '" not exists.');
    }

    /**
     * @inheritDoc
     */
    public function isBuildable()
    {
        if (!$this->isChecked) {
            $this->checkFunctions($this->buildMethods);
            $this->isChecked = true;
        }
        return empty($this->errors);
    }

    /**
     * @inheritDoc
     */
    public function getErrors()
    {
        return $this->errors;
    }

    /**
     * @inheritDoc
     */
    public function addError(\Exception $exception)
    {
        $this->errors[] = $exception;
        return $this;
    }
}
