<?php
namespace Setka\Editor\Service\SystemReport;

use Korobochkin\WPKit\Options\OptionInterface;
use Korobochkin\WPKit\PostMeta\PostMetaInterface;
use Korobochkin\WPKit\Transients\TransientInterface;

class SystemReport
{
    /**
     * @var SectionInterface[]
     */
    private $sections;

    /**
     * @var TypeConverterInterface
     */
    private $converter;

    /**
     * SystemReport constructor.
     *
     * @param SectionInterface[] $sections
     * @param $converter
     */
    public function __construct(array $sections, TypeConverterInterface $converter)
    {
        $this->sections  = $sections;
        $this->converter = $converter;
    }

    /**
     * @return \Generator
     */
    public function walk()
    {
        foreach ($this->sections as $section) {
            if ($section->isBuildable()) {
                try {
                    yield $section->getTitle() => $this->converter->convertToString($section->build());
                } catch (\Exception $exception) {
                    $section->addError($exception);
                    yield $section->getTitle() => $this->converter->convertToString($this->createArrayFromExceptions($section));
                }
            } else {
                yield $section->getTitle() => $this->converter->convertToString($this->createArrayFromExceptions($section));
            }
        }
    }

    /**
     * @param SectionInterface $section
     *
     * @return array
     */
    private function createArrayFromExceptions(SectionInterface $section)
    {
        $result = array();
        foreach ($section->getErrors() as $exception) {
            $result[] = $this->createArrayFromException($exception);
        }
        return $result;
    }

    /**
     * @param \Exception $exception
     *
     * @return array
     */
    private function createArrayFromException(\Exception $exception)
    {
        return array(
            'classname' => get_class($exception),
            'message' => $exception->getMessage(),
            'code' => $exception->getCode(),
            'line' => $exception->getLine(),
            'file' => $exception->getFile(),
        );
    }

    /**
     * @return string
     */
    public function buildFilename()
    {
        $url = parse_url($this->getSiteUrl());

        $name = array(
            'report',
            str_replace('.', '-', $url['host']),
        );

        if (!empty($url['path']) && '/' !== $url['path']) {
            $path = str_replace('/', '-', $url['path']);

            if ($path) {
                $name[] = $path;
            }
        }

        $name[] = time();

        return implode('-', $name);
    }

    /**
     * @return string
     */
    private function getSiteUrl()
    {
        return get_site_url();
    }
}
