<?php
namespace Setka\Editor\Admin\Service\FilesManager;

use Setka\Editor\Service\PostStatuses;
use Setka\Editor\Service\SetkaPostTypes;

class AssetsStatus
{
    const KEY_STATUS = 'status';

    const KEY_COUNTER = 'counter';

    /**
     * @var \wpdb
     */
    private $wpdb;

    /**
     * @var array
     */
    private static $countersTemplate = array(
        PostStatuses::ANY     => 0,
        PostStatuses::ARCHIVE => 0,
        PostStatuses::DRAFT   => 0,
        PostStatuses::PUBLISH => 0,
        PostStatuses::TRASH   => 0,
        PostStatuses::FUTURE  => 0,
        PostStatuses::PENDING => 0,
    );

    /**
     * AssetsStatus constructor.
     *
     * @param \wpdb $wpdb
     */
    public function __construct(\wpdb $wpdb)
    {
        $this->wpdb = $wpdb;
    }

    /**
     * @return array
     * @throws \Exception
     */
    public function getCounters()
    {
        $results = $this->castQueryResult($this->query(
            "SELECT
            post_status AS %s,
            COUNT(ID) AS %s
            FROM {$this->wpdb->posts}
            WHERE post_type = %s
            GROUP BY post_status",
            array(
                self::KEY_STATUS,
                self::KEY_COUNTER,
                SetkaPostTypes::FILE_POST_NAME,
            )
        ));

        return array_merge(self::$countersTemplate, $results, array(PostStatuses::ANY => $this->getTotalCounter()));
    }

    /**
     * @return int
     * @throws \Exception
     */
    public function getTotalCounter()
    {
        $results = $this->query(
            "SELECT COUNT(*) as %s FROM {$this->wpdb->posts} WHERE post_type = %s",
            array(
                self::KEY_COUNTER,
                SetkaPostTypes::FILE_POST_NAME,
            )
        );

        if (isset($results[0])) {
            return $this->castQuerySingle($results[0]);
        }
        throw $this->createUnknownException();
    }

    /**
     * @param string $query
     * @param array $replace
     *
     * @return array
     */
    private function query($query, array $replace)
    {
        // phpcs:disable WordPress.DB.PreparedSQL.NotPrepared
        return $this->wpdb->get_results($this->wpdb->prepare($query, $replace), ARRAY_A);
        // phpcs:enable
    }

    /**
     * @param mixed $results
     *
     * @return array
     *
     * @throws \Exception
     */
    private function castQueryResult($results)
    {
        if (!is_array($results)) {
            throw $this->createUnknownException();
        }

        if (empty($results)) {
            return $results;
        }

        $casted = array();

        foreach ($results as &$result) {
            if (isset($result[self::KEY_STATUS])) {
                $casted[$result[self::KEY_STATUS]] = $this->castQuerySingle($result);
            } else {
                throw $this->createUnknownException();
            }
        }

        return $casted;
    }

    /**
     * @param array $result
     *
     * @return int
     *
     * @throws \Exception
     */
    private function castQuerySingle(array $result)
    {
        if (isset($result['counter']) && is_numeric($result['counter'])) {
            return (int) $result['counter'];
        }
        throw $this->createUnknownException();
    }

    /**
     * @return \Exception
     */
    private function createUnknownException()
    {
        return new \Exception('Unknown result from $wpdb->query()');
    }
}
